package ibspan.tss.oms;

import com.hp.hpl.jena.ontology.OntModel;

import ibspan.tss.core.storage.ModelLoader;
import ibspan.tss.domain.vocabulary.RestaurantVocabulary;
import ibspan.tss.domain.vocabulary.RestaurantsDBVocabulary;
import ibspan.tss.oms.vocabulary.OMSVocabulary;
import ibspan.tss.oms.vocabulary.ExtInfoDBVocabulary;
import ibspan.tss.oms.vocabulary.OntologySourceDBVocabulary;

public class OMSModel {
	
	private OntModel mRestaurantsDB;
	private OntModel mRestaurantOnt;
	private OntModel mExtInfoDB;
	private OntModel mOntologySourceDB;
	private OntModel mOMSOnt;
	
	/** 
     * Creates a new instance of OMSModel 
     */
	private OMSModel() {
		mRestaurantsDB = null;
		mRestaurantOnt = null;
		mExtInfoDB = null;
		mOntologySourceDB = null;
		mOMSOnt = null;
	}
	
    /** 
     * Creates the unique instance of the OMSModel class
     */
    private static OMSModel instance = new OMSModel();
    
    /** 
     * Return the unique instance of the OMSModel class
     * @return the unique instance of the OMSModel class
     */
	public static synchronized OMSModel getInstance() {
		return OMSModel.instance;
	}
	
	public OntModel getRestaurantsDBModel() {
		if (mRestaurantsDB==null) {
			try
			{
				mRestaurantsDB = ModelLoader.loadDBModel(RestaurantsDBVocabulary.NS);
			} catch (Exception e){
				System.out.println("Load model RestaurantsDB Exception: " + e);
			} 
		}
		return mRestaurantsDB;
	}
	
	public OntModel getExtInfoDBModel() {
		if (mExtInfoDB==null) {
			try
			{
				mExtInfoDB = ModelLoader.loadDBModel(ExtInfoDBVocabulary.NS);
			} catch (Exception e){
				System.out.println("Load model ExtInfoDB Exception: " + e);
			} 
		}
		return mExtInfoDB;
	}
	
	public OntModel getOntologySourceDBModel() {
		if (mOntologySourceDB==null) {
			try
			{
				mOntologySourceDB = ModelLoader.loadDBModel(OntologySourceDBVocabulary.NS);
			} catch (Exception e){
				System.out.println("Load model OntologySourceDB Exception: " + e);
			} 
		}
		return mOntologySourceDB;
	}
	
	public OntModel getRestaurantOntModel() {
		if (mRestaurantOnt==null) {
			try
			{
				mRestaurantOnt = ModelLoader.loadModel(RestaurantVocabulary.NS);
			} catch (Exception e){
				System.out.println("Load model RestaurantOnt Exception: " + e);
			} 
		}
		return mRestaurantOnt;
	}
	
	public OntModel getOMSOntModel() {
		if (mOMSOnt==null) {
			try
			{
				mOMSOnt = ModelLoader.loadModel(OMSVocabulary.NS);
			} catch (Exception e){
				System.out.println("Load model OMSOnt Exception: " + e);
			} 
		}
		return mOMSOnt;
	}
}
