package ibspan.tss.oms;

import java.util.Iterator;

import org.mindswap.pellet.PelletOptions;
import org.mindswap.pellet.jena.PelletReasonerFactory;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.reasoner.ValidityReport;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Statement;

public class OMSReasoner {

	private OntModel modelToCheck;
	private ValidityReport validityReport;
	
	public OMSReasoner() {
		
		PelletOptions.USE_COMPLETION_QUEUE = true;
		PelletOptions.USE_INCREMENTAL_CONSISTENCY = true;
		PelletOptions.USE_SMART_RESTORE = false;
		
		this.modelToCheck = null;
	}
	
	public OMSReasoner(OntModel modelToCheck) {
		
		PelletOptions.USE_COMPLETION_QUEUE = true;
		PelletOptions.USE_INCREMENTAL_CONSISTENCY = true;
		PelletOptions.USE_SMART_RESTORE = false;
		
		this.modelToCheck = modelToCheck;
	}
	
	public void setModel(OntModel modelToCheck) {
		this.modelToCheck = modelToCheck;
	}
	
	public void addModel(OntModel modelToCheck) {
		this.modelToCheck.add(modelToCheck);
	}
	
	public void addStatement(Statement stmt) {
		this.modelToCheck.add(stmt);
	}
	
	public boolean checkConsistency() {
		return runReasoner();
	}
	
	public ValidityReport getValidityReport() {
		return validityReport;
	}
	
	public void printValidityReport() {
        for (Iterator i = validityReport.getReports(); i.hasNext(); ) {
            ValidityReport.Report report = (ValidityReport.Report)i.next();
            System.out.println(" - " + report);
        }
	}
	
	private boolean runReasoner() {
		
		OntModel model = ModelFactory.createOntologyModel(PelletReasonerFactory.THE_SPEC, modelToCheck);
		model.setStrictMode( false );
				
		// perform initial consistency check
		model.prepare();
		
	    ValidityReport validity = model.validate();
	    this.validityReport = validity;
	    if (validity.isValid()) {
	    	return true;
	    }
	    return false;
	}
}
