package ibspan.tss.oms.agents;

import java.util.HashMap;
import java.util.Map;

import com.hp.hpl.jena.ontology.OntModel;

import jade.core.Agent;
import jade.core.behaviours.CyclicBehaviour;
import jade.core.behaviours.ThreadedBehaviourFactory;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.UnreadableException;
import jade.util.Logger;

import ibspan.tss.oms.OMSCore;

public class OMSDBAgent extends Agent {
	
	private static Logger logger = Logger.getMyLogger(OMSDBAgent.class.getName());
	
	private ThreadedBehaviourFactory tbf = new ThreadedBehaviourFactory();
	
	private OMSCore omsCore;
	
	public OMSDBAgent() {
		omsCore = new OMSCore();
	}
	
	protected void setup() {
	    DFAgentDescription dfd = new DFAgentDescription();
	    dfd.setName(getAID());
	    ServiceDescription queryDB = new ServiceDescription();
	    queryDB.setType("DB-query");
	    queryDB.setName(getLocalName()+"-DB-query");
	    dfd.addServices(queryDB);
	    try {
	      DFService.register(this, dfd);
	    }
	    catch (FIPAException fe) {
	      fe.printStackTrace();
	    }
	    addBehaviour(tbf.wrap(new GetModelsForUpdate()));
	    addBehaviour(tbf.wrap(new GetModelRestaurantOnt()));
	    addBehaviour(tbf.wrap(new CompareUpdate()));
	    addBehaviour(tbf.wrap(new NewDataIndexing()));
	    addBehaviour(tbf.wrap(new GetIncompleteData()));
	    addBehaviour(tbf.wrap(new InsertIncompleteData()));
	}
	
	private class GetModelsForUpdate extends CyclicBehaviour {		
		private Map<String, Map<Long, String>> mapSendModels;
		private MessageTemplate mt = MessageTemplate.and(
				MessageTemplate.MatchPerformative(ACLMessage.REQUEST),
				MessageTemplate.MatchConversationId("OMSgetModelsForUpdate")
				);
		public void action() {			
			ACLMessage msg = myAgent.receive(mt);
			if (msg!=null) {
				mapSendModels = omsCore.createModelsForUpdate(Integer.parseInt(msg.getContent()));
				ACLMessage reply = msg.createReply();
				if (mapSendModels.isEmpty()) {
					reply.setPerformative(ACLMessage.REFUSE);
				}
				else {
					reply.setPerformative(ACLMessage.INFORM);
					try {
						reply.setContentObject((java.io.Serializable)mapSendModels);
					}
					catch (Exception e) { 
						e.printStackTrace(); 
					}
				}
				myAgent.send(reply);
			}
			else {
				block();
			}
		}
	}
	
	private class GetModelRestaurantOnt extends CyclicBehaviour {
		private MessageTemplate mt = MessageTemplate.and(
				MessageTemplate.MatchPerformative(ACLMessage.REQUEST),
				MessageTemplate.MatchConversationId("OMSgetModelRestaurantOnt")
				);
		public void action() {
			ACLMessage msg = myAgent.receive(mt);
			if (msg!=null) {
				logger.info(myAgent.getLocalName() + "- message received");
				ACLMessage reply = msg.createReply();
				String restOnt = omsCore.getModelRestaurantOntRDF();
				if (restOnt.length()>0) {
					reply.setPerformative(ACLMessage.INFORM);
					try {
						reply.setContentObject((java.io.Serializable)restOnt);
					} catch (Exception e) { 
						e.printStackTrace(); 
					}
				}
				else {
					reply.setPerformative(ACLMessage.REFUSE);
				}
				myAgent.send(reply);
				logger.info(myAgent.getLocalName() + "- message responded");
			}
			else {
				block();
			}
		}
	}
	
	private class CompareUpdate extends CyclicBehaviour {
		private MessageTemplate mt = MessageTemplate.and(
				MessageTemplate.MatchPerformative(ACLMessage.REQUEST),
				MessageTemplate.MatchConversationId("OMScompareUpdate")
				);
		public void action() {
			ACLMessage msg = myAgent.receive(mt);
			if (msg!=null) {
				logger.info(myAgent.getLocalName() + "- message received");
				boolean comparisonDone = false;
				int cntUpdates = 0;
				Map<String, String> mapUpdate = null;
				try {
					mapUpdate = (Map<String, String>) msg.getContentObject();
				} catch (UnreadableException e) {
					logger.info(myAgent.getLocalName() + "Error: " + e.toString());
				}
				for (Map.Entry<String, String> entry : mapUpdate.entrySet()) {
					try {
						cntUpdates = omsCore.compareRequestResponseModels(omsCore.rdf2model(entry.getKey()), 
							omsCore.rdf2model(entry.getValue()));
						comparisonDone = true;
					} catch (Exception e) {
						comparisonDone = false;
						e.printStackTrace();
					}
				}
				ACLMessage reply = msg.createReply();
				if (comparisonDone) {
					reply.setPerformative(ACLMessage.CONFIRM);
				}
				else {
					reply.setPerformative(ACLMessage.FAILURE);
				}				
				myAgent.send(reply);
				logger.info(myAgent.getLocalName() + "- message responded");
			}
			else {
				block();
			}
		}
	}
	
	private class NewDataIndexing extends CyclicBehaviour {
		private MessageTemplate mt = MessageTemplate.and(
				MessageTemplate.MatchPerformative(ACLMessage.INFORM),
				MessageTemplate.MatchConversationId("OMSNewDataIndexing")
				);
		int cntNewData=0;
		public void action() {
			ACLMessage msg = myAgent.receive(mt);
			if (msg!=null) {
				logger.info(myAgent.getLocalName() + "- new data received");
				try {
					cntNewData = omsCore.newDataIndexing((Map<String, String>) msg.getContentObject());
					logger.info(myAgent.getLocalName() + "- new added statements = " + cntNewData);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			else {
				block();
			}
		}
	}
	
	private class GetIncompleteData extends CyclicBehaviour {		
		private Map<String, Map<Long, String>> mapSendModels;
		private MessageTemplate mt = MessageTemplate.and(
				MessageTemplate.MatchPerformative(ACLMessage.REQUEST),
				MessageTemplate.MatchConversationId("OMSgetIncompleteData")
				);
		public void action() {			
			ACLMessage msg = myAgent.receive(mt);
			if (msg!=null) {
				mapSendModels = omsCore.createIncompleteModels(Integer.parseInt(msg.getContent()));
				ACLMessage reply = msg.createReply();
				if (mapSendModels.isEmpty()) {
					reply.setPerformative(ACLMessage.REFUSE);
				}
				else {
					reply.setPerformative(ACLMessage.INFORM);
					try {
						reply.setContentObject((java.io.Serializable)mapSendModels);
					}
					catch (Exception e) { 
						e.printStackTrace(); 
					}
				}
				myAgent.send(reply);
			}
			else {
				block();
			}
		}
	}
	
	private class InsertIncompleteData extends CyclicBehaviour {
		private MessageTemplate mt = MessageTemplate.and(
				MessageTemplate.MatchPerformative(ACLMessage.REQUEST),
				MessageTemplate.MatchConversationId("OMSinsertIncompleteData")
				);
		public void action() {
			ACLMessage msg = myAgent.receive(mt);
			if (msg!=null) {
				logger.info(myAgent.getLocalName() + "- message received");
				boolean done = false;
				int cnt = 0;
				Map<String, String> mapIncomplete = null;
				try {
					mapIncomplete = (Map<String, String>) msg.getContentObject();
				} catch (UnreadableException e) {
					logger.info(myAgent.getLocalName() + "Error: " + e.toString());
				}
				for (Map.Entry<String, String> entry : mapIncomplete.entrySet()) {
					try {
						cnt = omsCore.insertIncompleteData(entry.getKey(), 
								omsCore.rdf2model(entry.getValue()));
						logger.info(myAgent.getLocalName() + "- new added statements = " + cnt);
						done = true;
					} catch (Exception e) {
						done = false;
						e.printStackTrace();
					}
				}
				ACLMessage reply = msg.createReply();
				if (done) {
					reply.setPerformative(ACLMessage.CONFIRM);
				}
				else {
					reply.setPerformative(ACLMessage.FAILURE);
				}				
				myAgent.send(reply);
				logger.info(myAgent.getLocalName() + "- message responded");
			}
			else {
				block();
			}
		}
	}

}
