package ibspan.tss.oms.agents;

import java.io.IOException;
import java.util.Map;
import java.util.HashMap;
import java.util.Properties;
import java.util.Set;
import java.util.HashSet;
import java.util.Vector;

import jade.core.AID;
import jade.core.Agent;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.CyclicBehaviour;
import jade.core.behaviours.OneShotBehaviour;
import jade.core.behaviours.ThreadedBehaviourFactory;
import jade.core.behaviours.TickerBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.UnreadableException;
import jade.util.Logger;
import jade.core.*;
import jade.domain.*;
import jade.domain.JADEAgentManagement.*;
import jade.content.onto.basic.*;


public final class OMSIncompleteDataAgent extends Agent {

	private static Logger logger = Logger.getMyLogger(OMSIncompleteDataAgent.class.getName());
	
	private static final String OMS_CONF_FILE = "file:etc/oms-conf.properties";
	
	private final int FREQ_CHECK;
	private final int CNT_TRIPP_INCOMPLETE;
	
	private ThreadedBehaviourFactory tbf = new ThreadedBehaviourFactory();
	
	private Vector<AID> dbAgents = new Vector<AID>();
	
	Behaviour b;
	
	public OMSIncompleteDataAgent() {
		Properties props = loadProperties(OMS_CONF_FILE);
		FREQ_CHECK = Integer.valueOf(props.getProperty("omsIDAFrequencyChecking"));
		CNT_TRIPP_INCOMPLETE = Integer.valueOf(props.getProperty("omsIDAMaxCntStatements"));
	}
	
	protected void setup() {
				
	    getContentManager().registerLanguage(new jade.content.lang.sl.SLCodec(0));
	    getContentManager().registerOntology(JADEManagementOntology.getInstance());
		
		addBehaviour(new OneShotBehaviour() {
			public void action() {
				DFAgentDescription dfd = new DFAgentDescription();
				ServiceDescription sd = new ServiceDescription();
				sd.setType("DB-query");
				dfd.addServices(sd);
		        try {
		        	DFAgentDescription[] result = DFService.search(myAgent, dfd);
		            dbAgents.clear();
		            for (int i = 0; i < result.length; ++i) {
		            	dbAgents.addElement(result[i].getName());
		            }
		        }
		        catch (FIPAException fe) {
		        	fe.printStackTrace();
		        }
			}
		});
		
		b = new GetIncompleteData(CNT_TRIPP_INCOMPLETE);
		addBehaviour(tbf.wrap(b));
		
		addBehaviour(tbf.wrap(new CyclicBehaviour() {
			public void action() {
				if (b.done()) {
					b = new GetIncompleteData(CNT_TRIPP_INCOMPLETE);
					addBehaviour(tbf.wrap(b));
				    try {
				    	Thread.sleep(FREQ_CHECK);
					} catch (InterruptedException ingore) {}
					block();
				}
				else {
				    try {
				    	Thread.sleep(FREQ_CHECK);
					} catch (InterruptedException ingore) {}
					block();
				}
			}
		}));
		    
	}
	    
	private class GetIncompleteData extends Behaviour {
		private CreateAgent ca;
		private Map<String, AID> searchAgents;
		private MessageTemplate mt;
		private Map<String, Map<Long, String>> mapCurrentModels;
		private Set<MessageTemplate> listMT;
		private int step;
		private int stepEnd;
		private int cntSearchAgents;
		private int cntTripplesIncomplete;
		
		public GetIncompleteData(int cntTripplesIncomplete) {
			this.cntTripplesIncomplete = cntTripplesIncomplete;
			cntSearchAgents = 0;
			step = 0;
			stepEnd = 6;
			searchAgents = new HashMap<String, AID>();
			mapCurrentModels = null;
			listMT = new HashSet<MessageTemplate>();
		}
		
		public void action() {
			switch (step) {
			case 0:		
		        ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
		        for (int i = 0; i < dbAgents.size(); ++i) {
		        	msg.addReceiver((AID)dbAgents.elementAt(i));
		        }
		        msg.setContent(Integer.toString(this.cntTripplesIncomplete));
		        msg.setConversationId("OMSgetIncompleteData");
		        msg.setReplyWith("incomplete"+System.currentTimeMillis());
		        myAgent.send(msg);
		        mt = MessageTemplate.and(
		        		MessageTemplate.MatchConversationId(msg.getConversationId()), 
		        		MessageTemplate.MatchInReplyTo(msg.getReplyWith()));
		        step=1;
		        break;
			case 1: 
		        ACLMessage reply = myAgent.receive(mt);
		        if (reply!=null) {
		        	if (reply.getPerformative() == ACLMessage.REFUSE) {
		        		logger.info(myAgent.getLocalName() + "- no incomplete data");
		        		step=stepEnd;
		        	}
		        	else {
			        	try {
			        		mapCurrentModels = (Map<String, Map<Long, String>>) reply.getContentObject();
						} catch (Exception e) { 
							e.printStackTrace(); 
						}
						step=2;
		        	}
		        }
		        else {
		        	block();
		        }
		        break;
			case 2:
				for (Map.Entry<String, Map<Long, String>> entry : mapCurrentModels.entrySet()) {
					for (Map.Entry<Long, String> entry2 : entry.getValue().entrySet()) {
					    ca = new CreateAgent();
					    ca.setAgentName(entry.getKey().toString() + "_" + entry2.getKey().toString());
					    ca.setClassName("ibspan.tss.oms.agents.OMSSearchAgent");
					    
					    // add as argument the local name of this agent
					    ca.addArguments(myAgent.getLocalName());
					    // add URL of ontology source as argument of agent
					    ca.addArguments(entry.getKey());
					    
					    ca.setContainer(new ContainerID(AgentContainer.MAIN_CONTAINER_NAME, null));
					    Action a = new Action();
					    a.setActor(getAMS());
					    a.setAction(ca);
					    ACLMessage m = new ACLMessage(ACLMessage.REQUEST);
					    m.setOntology(JADEManagementOntology.getInstance().getName());
					    m.setSender(new AID(getLocalName(), AID.ISLOCALNAME));
					    m.setProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST);
					    m.setLanguage(FIPANames.ContentLanguage.FIPA_SL0);
					    m.addReceiver(getAMS());
					    try {
					        getContentManager().fillContent(m, a);
					    } catch (Exception e) {
						      e.printStackTrace();
					    }
					    myAgent.send(m);					    
				        AID searchA = new AID((String)ca.getAgentName(), AID.ISLOCALNAME);
				        searchAgents.put(ca.getAgentName(), searchA);
					}
				}
				cntSearchAgents = 0;
				step=3;
				break;
			case 3:
				if (cntSearchAgents==searchAgents.size())
				{
					logger.info(myAgent.getLocalName() + "- all search agents created, count agents=" + cntSearchAgents);
					step=4;
				} else {
				    ACLMessage msgAgentCreated = myAgent.receive(MessageTemplate.and(
				    		MessageTemplate.MatchPerformative(ACLMessage.INFORM),
				    		MessageTemplate.MatchConversationId("SearchAgentCreated")));
				    if (msgAgentCreated!=null) {
				    	cntSearchAgents++;
				    	step=3;
					}
					else {
						block();
					}
				}
				break;
			case 4:
				for (Map.Entry<String, Map<Long, String>> entry : mapCurrentModels.entrySet()) {
					for (Map.Entry<Long, String> entry2 : entry.getValue().entrySet()) {
				        ACLMessage msgSearch = new ACLMessage(ACLMessage.REQUEST);
				        msgSearch.addReceiver(searchAgents.get(entry.getKey().toString() + "_" + entry2.getKey().toString()));
				        // URL of search page
				        msgSearch.setProtocol(entry.getKey().toString());
				        // unique value (long)
				        msgSearch.setOntology(entry2.getKey().toString());
				        try {
				        	// ontology model for checking updates
				        	msgSearch.setContentObject(entry2.getValue());
						} catch (IOException e) {
							logger.info(myAgent.getLocalName() + "Error: " + e.toString());
						}
				        msgSearch.setConversationId("OMSsearchData");
				        msgSearch.setReplyWith("search"+ entry.getKey().toString() + entry2.getKey().toString());
				        myAgent.send(msgSearch);
				        mt = MessageTemplate.and(
				        		MessageTemplate.MatchConversationId(msgSearch.getConversationId()), 
				        		MessageTemplate.MatchInReplyTo(msgSearch.getReplyWith()));
				        listMT.add(mt);
					}
				}
				cntSearchAgents = 0;
				step=5;
				break;
			case 5:
				ACLMessage msgReceiveUpdate = null;
				for (MessageTemplate mesTemp : listMT) {
				    msgReceiveUpdate = myAgent.receive(mesTemp);
				    if (msgReceiveUpdate!=null) {
				    	listMT.remove(mesTemp);
				    	break;
				    }
				}
			    if (msgReceiveUpdate!=null) {
			    	cntSearchAgents++;
			        ACLMessage msgIncompleteData = new ACLMessage(ACLMessage.REQUEST);
			        for (int i = 0; i < dbAgents.size(); ++i) {
			        	msgIncompleteData.addReceiver((AID)dbAgents.elementAt(i));
			        }
			        msgIncompleteData.setConversationId("OMSinsertIncompleteData");
			        // URL of search page
			        msgIncompleteData.setProtocol(msgReceiveUpdate.getProtocol());
			        // unique value (long)
			        msgIncompleteData.setOntology(msgReceiveUpdate.getOntology());
			        try {
			        	// ontology model for checking updates
			        	Map<String, String> mapUpdate = new HashMap<String, String>();
			        	mapUpdate.put(msgReceiveUpdate.getProtocol(), 
			        			msgReceiveUpdate.getContentObject().toString());
			        	msgIncompleteData.setContentObject((java.io.Serializable)mapUpdate);
					} catch (IOException e) {
						logger.info(myAgent.getLocalName() + "Error: " + e.toString());
					} catch (UnreadableException e) {
						logger.info(myAgent.getLocalName() + "Error: " + e.toString());
					}
			        msgIncompleteData.setReplyWith("compare"+msgReceiveUpdate.getContent()+"_"+
			        		msgReceiveUpdate.getOntology());
			        myAgent.send(msgIncompleteData);
			        mt = MessageTemplate.and(
				        	MessageTemplate.MatchConversationId(msgIncompleteData.getConversationId()), 
			        		MessageTemplate.MatchInReplyTo(msgIncompleteData.getReplyWith()));
			        addBehaviour( new CyclicBehaviour() {
			        	public void action () {
					        ACLMessage replyOnt = myAgent.receive(mt);
					        if (replyOnt!=null) {
					        	if (replyOnt.getPerformative() == ACLMessage.FAILURE) {
					        		logger.info(myAgent.getLocalName() + "- incomplete data not inserted");
					        	}
					        	else {
					        		logger.info(myAgent.getLocalName() + "- incomplete data inserted");
					        	}
								if (cntSearchAgents==searchAgents.size())
								{
									logger.info(myAgent.getLocalName() + "- all incomplete data received");
									step=stepEnd;
								}
					        }
					        else {
					        	block();
					        }
			        	}
			        });
			        logger.info(myAgent.getLocalName() + "- incomplete data received");
			        step=5;
				}
				else {
					block();
				}
				break;
			}
		}
		public boolean done() {
			return step==stepEnd;
		}
	}
	
	// Loads properties from the pointed URL.
	private static Properties loadProperties(String confURL) {
		Properties conf = null;
		try {
			java.net.URL url = new java.net.URL(confURL);
			conf = new Properties();
			conf.load(url.openStream());
		} catch (IOException e) {
			throw new RuntimeException("Cannot get properties from configuration file.", e);
		}
		return conf;
	}
}
