

package ibspan.tss.oms.ontology;

import com.ibm.adtech.jastor.*;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.vocabulary.RDF;

/**
 * Factory for instantiating objects for ontology classes in the OMS ontology.  The
 * get methods leave the model unchanged and return a Java view of the object in the model.  The create methods
 * may add certain baseline properties to the model such as rdf:type and any properties with hasValue restrictions.
 * <p>(URI: http://www.ibspan.waw.pl/tss/OMS)</p>
 * <br>
 * <br>
 * <br>
 */
public class OMSFactory extends com.ibm.adtech.jastor.ThingFactory { 



	/**
	 * Create a new instance of ExtInfo.  Adds the rdf:type property for the given resource to the model.
	 * @param resource The resource of the ExtInfo
	 * @param model the Jena Model.
	 */
	public static ExtInfo createExtInfo(Resource resource, Model model) throws JastorException {
		return ibspan.tss.oms.ontology.ExtInfoImpl.createExtInfo(resource,model);
	}
	
	/**
	 * Create a new instance of ExtInfo.  Adds the rdf:type property for the given resource to the model.
	 * @param uri The uri of the ExtInfo
	 * @param model the Jena Model.
	 */
	public static ExtInfo createExtInfo(String uri, Model model) throws JastorException {
		ExtInfo obj = ibspan.tss.oms.ontology.ExtInfoImpl.createExtInfo(model.createResource(uri), model);
		return obj;
	}
	
	/**
	 * Create a new instance of ExtInfo.  Leaves the model unchanged.
	 * @param uri The uri of the ExtInfo
	 * @param model the Jena Model.
	 */
	public static ExtInfo getExtInfo(String uri, Model model) throws JastorException {
		return getExtInfo(model.createResource(uri),model);
	}
	
	/**
	 * Create a new instance of ExtInfo.  Leaves the model unchanged.
	 * @param resource The resource of the ExtInfo
	 * @param model the Jena Model.
	 */
	public static ExtInfo getExtInfo(Resource resource, Model model) throws JastorException {
		String code = (model.hashCode()*17 + ibspan.tss.oms.ontology.ExtInfo.class.hashCode()) + resource.toString();
		ibspan.tss.oms.ontology.ExtInfoImpl obj = (ibspan.tss.oms.ontology.ExtInfoImpl)objects.get(code);
		if (obj == null) {
			obj = ibspan.tss.oms.ontology.ExtInfoImpl.getExtInfo(resource, model);
			if (obj == null)
				return null;
			objects.put(code, obj);
		}
		return obj;
	}
	
	/**
	 * Return an instance of ExtInfo for every resource in the model with rdf:Type http://www.ibspan.waw.pl/tss/OMS#ExtInfo
	 * @param model the Jena Model
	 * @return a List of ExtInfo
	 */
	public static java.util.List getAllExtInfo(Model model) throws JastorException {
		StmtIterator it = model.listStatements(null,RDF.type,ExtInfo.TYPE);
		java.util.List list = new java.util.ArrayList();
		while (it.hasNext()) {
			Statement stmt = it.nextStatement();
			list.add(getExtInfo(stmt.getSubject(),model));
		}
		return list;
	}
	

	/**
	 * Create a new instance of OntologySource.  Adds the rdf:type property for the given resource to the model.
	 * @param resource The resource of the OntologySource
	 * @param model the Jena Model.
	 */
	public static OntologySource createOntologySource(Resource resource, Model model) throws JastorException {
		return ibspan.tss.oms.ontology.OntologySourceImpl.createOntologySource(resource,model);
	}
	
	/**
	 * Create a new instance of OntologySource.  Adds the rdf:type property for the given resource to the model.
	 * @param uri The uri of the OntologySource
	 * @param model the Jena Model.
	 */
	public static OntologySource createOntologySource(String uri, Model model) throws JastorException {
		OntologySource obj = ibspan.tss.oms.ontology.OntologySourceImpl.createOntologySource(model.createResource(uri), model);
		return obj;
	}
	
	/**
	 * Create a new instance of OntologySource.  Leaves the model unchanged.
	 * @param uri The uri of the OntologySource
	 * @param model the Jena Model.
	 */
	public static OntologySource getOntologySource(String uri, Model model) throws JastorException {
		return getOntologySource(model.createResource(uri),model);
	}
	
	/**
	 * Create a new instance of OntologySource.  Leaves the model unchanged.
	 * @param resource The resource of the OntologySource
	 * @param model the Jena Model.
	 */
	public static OntologySource getOntologySource(Resource resource, Model model) throws JastorException {
		String code = (model.hashCode()*17 + ibspan.tss.oms.ontology.OntologySource.class.hashCode()) + resource.toString();
		ibspan.tss.oms.ontology.OntologySourceImpl obj = (ibspan.tss.oms.ontology.OntologySourceImpl)objects.get(code);
		if (obj == null) {
			obj = ibspan.tss.oms.ontology.OntologySourceImpl.getOntologySource(resource, model);
			if (obj == null)
				return null;
			objects.put(code, obj);
		}
		return obj;
	}
	
	/**
	 * Return an instance of OntologySource for every resource in the model with rdf:Type http://www.ibspan.waw.pl/tss/OMS#OntologySource
	 * @param model the Jena Model
	 * @return a List of OntologySource
	 */
	public static java.util.List getAllOntologySource(Model model) throws JastorException {
		StmtIterator it = model.listStatements(null,RDF.type,OntologySource.TYPE);
		java.util.List list = new java.util.ArrayList();
		while (it.hasNext()) {
			Statement stmt = it.nextStatement();
			list.add(getOntologySource(stmt.getSubject(),model));
		}
		return list;
	}
	
	
	/**
	 * Returns an instance of an interface for the given Resource.  The return instance is guaranteed to 
	 * implement the most specific interface in *some* hierarchy in which the Resource participates.  The behavior
	 * is unspecified for resources with RDF types from different hierarchies.
	 * @return an instance of Thing
	 */
	public static Thing getThing(com.hp.hpl.jena.rdf.model.Resource res, com.hp.hpl.jena.rdf.model.Model model) throws JastorException {
		if (res.hasProperty(RDF.type,model.getResource("http://www.ibspan.waw.pl/tss/OMS#OntologySource"))) {
			return getOntologySource(res,model);
		}
		if (res.hasProperty(RDF.type,model.getResource("http://www.ibspan.waw.pl/tss/OMS#ExtInfo"))) {
			return getExtInfo(res,model);
		}
		return new ThingImpl(res,model);
	}
	
	/**
	 * Returns an instance of an interface for the given Resource URI.  The return instance is guaranteed to 
	 * implement the most specific interface in *some* hierarchy in which the Resource participates.  The behavior
	 * is unspecified for resources with RDF types from different hierarchies.
	 * @return an instance of Thing
	 */
	public static Thing getThing(String uri, com.hp.hpl.jena.rdf.model.Model model) throws JastorException {
		return getThing(model.getResource(uri),model);
	}

	/**
	 * Return a list of compatible interfaces for the given type.  Searches through all ontology classes
	 * in the OMS ontology.  The list is sorted according to the topological sort
	 * of the class hierarchy
	 * @return a List of type java.lang.Class
	 */
	public static java.util.List listCompatibleInterfaces (com.hp.hpl.jena.rdf.model.Resource type) {
		java.util.List types = new java.util.ArrayList();
		if (type.equals(ibspan.tss.oms.ontology.OntologySource.TYPE)) {
			types.add(ibspan.tss.oms.ontology.OntologySource.class);
		}
		if (type.equals(ibspan.tss.oms.ontology.ExtInfo.TYPE)) {
			types.add(ibspan.tss.oms.ontology.ExtInfo.class);
		}
		return types;
	}
}